open List

(* I use these functions a lot. They are like a standard base for
   me. You need to know them in order to understand my code *)
module MyUtils = struct 
  type 'a find_result = NotFound | Found of 'a

  let rec find l p = 
    match l with
	[] -> NotFound
      | h::t ->  if p h then Found h else find t p

  (* I am inverting the order of the args in list.exists and
     List.filter. The list is now the first argument, because it is
     usually shorter than the second. So I can write stuff like

     exists l1 (fun x -> exists l2 (fun y -> x + y = 3))

     which is readable. Using List.exists it would have been
     unreadable.

     It is the same cognitive phenomenon that makes you understand the
     sentence:

     "I read the book that (is on the table that (was built by the man
     that (was here yesterday)))"

     and not the sentence

     "The man (that the father (that the brother knew) saw) ate" *)
  let exists l p = List.exists p l

  let forall l p =  List.for_all p l

  let setof l p = List.filter p l

  let between x y z =  y <= x && x <= z

  let number_of l p =  length (setof l p)

  let rec pairs x y = 
    match x with
	x1::xt -> 
	  let x1y = List.map (fun k -> (x1, k)) y 
	  in
	  List.append x1y (pairs xt y)
      | [] -> []

  let remove x l eq = setof l (fun o -> not(eq o x))

  (* "accumulate l f state" invokes f on each element of l, but
     threading a state. This means that

     f x state

     is called for each x in l, but each time with the state produced
     by the previous call.

     Example:

     accumulate [1;2;4] f initial_state

     means

     let st1 = f 1 initial_state in
     let st2 = f 2 st1 in
     f 4 st2

  *)

  let rec accumulate ~list ~f ~state =
    match list with
	[] -> state
      | h::t -> 
	  let state' = f h state 
	  in
	  accumulate ~list:t ~f:f ~state:state'

  let rec replace l o o' eq = 
    match l with
	[] -> []
      | h::t ->
	  if eq h o then
	    o' :: replace t o o' eq
	  else
	    h :: replace t o o' eq	      

  exception EmptyList

  let rec minimum x = 
    match x with
	[a] -> a 
      | h::t -> min h (minimum t)
      | [] -> raise EmptyList
	  
end

open MyUtils

type event =   Timer of int  |  Resize of int * int  
	       | Redraw |  Init |   MouseMoved of float*float  
	       | MouseClicked of int * bool

let pending_events = ref []


let get_pending_events () = 
  let exit = Glib.Main.iteration true in
  let p = !pending_events  in
  pending_events := [];
  p

let main () =
  
  let delete_event ev =
    print_endline "Delete event occurred";
    flush stdout;
    false (* false = quit *)  in
  
  let timer_callback millisec () = 
    pending_events := append !pending_events [Timer millisec];
    true in
  let mouse_press_callback ev =
    pending_events := append !pending_events [MouseClicked (GdkEvent.Button.button ev , true)];
    true (* event handled, don't propagate *) in
  let mouse_release_callback ev =
    pending_events := append !pending_events [MouseClicked (GdkEvent.Button.button ev , false)];
    true (* event handled, don't propagate *) in
  let mouse_motion_callback ev =
    pending_events := append !pending_events
      [MouseMoved  (GdkEvent.Motion.x ev, GdkEvent.Motion.y ev) ];
    true (* event handled, don't propagate *) in
  let reshape_callback ~width ~height = 
    pending_events := append !pending_events [Resize (width, height)] in
  let realize_callback () = 
    pending_events := append !pending_events [Init] in
  let display_callback () = 
    pending_events := append !pending_events [Redraw] in

  let destroy () = print_endline "destroy"; GMain.Main.quit () in
  let locale = GtkMain.Main.init () in
  let window = GWindow.window  () in
  let _ = window#event#connect#delete ~callback:delete_event in
  let _ = window#connect#destroy ~callback:destroy in
  window#maximize ();
  window#show ()



let _ = main () 
